@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Zend OPcache                                                         |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1998-2017 The PHP Group                                |
+   | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -12,19 +12,19 @@
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
    +----------------------------------------------------------------------+
-   | Authors: Andi Gutmans <andi@zend.com>                                |
-   |          Zeev Suraski <zeev@zend.com>                                |
+   | Authors: Andi Gutmans <andi@php.net>                                 |
+   |          Zeev Suraski <zeev@php.net>                                 |
    |          Stanislav Malyshev <stas@zend.com>                          |
-   |          Dmitry Stogov <dmitry@zend.com>                             |
+   |          Dmitry Stogov <dmitry@php.net>                              |
    +----------------------------------------------------------------------+
 */
 
 /* pass 1
  * - substitute persistent constants (true, false, null, etc)
  * - perform compile-time evaluation of constant binary and unary operations
- * - optimize series of ADD_STRING and/or ADD_CHAR
  * - convert CAST(IS_BOOL,x) into BOOL(x)
  * - pre-evaluate constant function calls
+ * - eliminate FETCH $GLOBALS followed by FETCH_DIM/UNSET_DIM/ISSET_ISEMPTY_DIM
  */
 
 #include "php.h"
@@ -35,8 +35,6 @@
 #include "zend_execute.h"
 #include "zend_vm.h"
 
-#define ZEND_IS_CONSTANT_TYPE(t)	((t) == IS_CONSTANT)
-
 void zend_optimizer_pass1(zend_op_array *op_array, zend_optimizer_ctx *ctx)
 {
 	int i = 0;
@@ -68,79 +66,45 @@ void zend_optimizer_pass1(zend_op_array *op_array, zend_optimizer_ctx *ctx)
 		case ZEND_BW_XOR:
 		case ZEND_BOOL_XOR:
 		case ZEND_SPACESHIP:
-			if (ZEND_OP1_TYPE(opline) == IS_CONST &&
-				ZEND_OP2_TYPE(opline) == IS_CONST) {
+		case ZEND_CASE:
+			if (opline->op1_type == IS_CONST &&
+				opline->op2_type == IS_CONST) {
 				/* binary operation with constant operands */
-				binary_op_type binary_op = get_binary_op(opline->opcode);
-				uint32_t tv = ZEND_RESULT(opline).var;		/* temporary variable */
 				zval result;
-				int er;
 
-				if ((opline->opcode == ZEND_DIV || opline->opcode == ZEND_MOD) &&
-					zval_get_long(&ZEND_OP2_LITERAL(opline)) == 0) {
-					/* div by 0 */
-					break;
-				} else if ((opline->opcode == ZEND_SL || opline->opcode == ZEND_SR) &&
-					zval_get_long(&ZEND_OP2_LITERAL(opline)) < 0) {
-					/* shift by negative number */
-					break;
-				} else if (zend_binary_op_produces_numeric_string_error(opline->opcode, &ZEND_OP1_LITERAL(opline), &ZEND_OP2_LITERAL(opline))) {
-					/* produces numeric string E_NOTICE/E_WARNING */
-					break;
-				}
-				er = EG(error_reporting);
-				EG(error_reporting) = 0;
-				/* evaluate constant expression */
-				if (binary_op(&result, &ZEND_OP1_LITERAL(opline), &ZEND_OP2_LITERAL(opline)) != SUCCESS) {
-					EG(error_reporting) = er;
-					break;
-				}
-				EG(error_reporting) = er;
-
-				if (zend_optimizer_replace_by_const(op_array, opline + 1, IS_TMP_VAR, tv, &result)) {
+				if (zend_optimizer_eval_binary_op(&result, opline->opcode, &ZEND_OP1_LITERAL(opline), &ZEND_OP2_LITERAL(opline)) == SUCCESS) {
 					literal_dtor(&ZEND_OP1_LITERAL(opline));
 					literal_dtor(&ZEND_OP2_LITERAL(opline));
-					MAKE_NOP(opline);
+					if (zend_optimizer_replace_by_const(op_array, opline + 1, IS_TMP_VAR, opline->result.var, &result)) {
+						MAKE_NOP(opline);
+					} else {
+						opline->opcode = ZEND_QM_ASSIGN;
+						SET_UNUSED(opline->op2);
+						zend_optimizer_update_op1_const(op_array, opline, &result);
+					}
 				}
 			}
 			break;
 
 		case ZEND_CAST:
-			if (ZEND_OP1_TYPE(opline) == IS_CONST &&
-				opline->extended_value != IS_ARRAY &&
-				opline->extended_value != IS_OBJECT &&
-				opline->extended_value != IS_RESOURCE &&
-				(opline->extended_value != IS_STRING
-					|| Z_TYPE(ZEND_OP1_LITERAL(opline)) != IS_ARRAY)) {
+			if (opline->op1_type == IS_CONST) {
 				/* cast of constant operand */
-				zend_uchar type = opline->result_type;
-				uint32_t tv = ZEND_RESULT(opline).var;		/* temporary variable */
-				zval res;
-				res = ZEND_OP1_LITERAL(opline);
-				zval_copy_ctor(&res);
-				switch (opline->extended_value) {
-					case IS_NULL:
-						convert_to_null(&res);
-						break;
-					case _IS_BOOL:
-						convert_to_boolean(&res);
-						break;
-					case IS_LONG:
-						convert_to_long(&res);
-						break;
-					case IS_DOUBLE:
-						convert_to_double(&res);
-						break;
-					case IS_STRING:
-						convert_to_string(&res);
-						break;
-				}
+				zval result;
 
-				if (zend_optimizer_replace_by_const(op_array, opline + 1, type, tv, &res)) {
+				if (zend_optimizer_eval_cast(&result, opline->extended_value, &ZEND_OP1_LITERAL(opline)) == SUCCESS) {
 					literal_dtor(&ZEND_OP1_LITERAL(opline));
-					MAKE_NOP(opline);
+					if (zend_optimizer_replace_by_const(op_array, opline + 1, opline->result_type, opline->result.var, &result)) {
+						MAKE_NOP(opline);
+					} else {
+						opline->opcode = ZEND_QM_ASSIGN;
+						opline->extended_value = 0;
+						zend_optimizer_update_op1_const(op_array, opline, &result);
+					}
+					break;
 				}
-			} else if (opline->extended_value == _IS_BOOL) {
+			}
+
+			if (opline->extended_value == _IS_BOOL) {
 				/* T = CAST(X, IS_BOOL) => T = BOOL(X) */
 				opline->opcode = ZEND_BOOL;
 				opline->extended_value = 0;
@@ -149,103 +113,24 @@ void zend_optimizer_pass1(zend_op_array *op_array, zend_optimizer_ctx *ctx)
 
 		case ZEND_BW_NOT:
 		case ZEND_BOOL_NOT:
-			if (ZEND_OP1_TYPE(opline) == IS_CONST) {
+			if (opline->op1_type == IS_CONST) {
 				/* unary operation on constant operand */
-				unary_op_type unary_op = get_unary_op(opline->opcode);
 				zval result;
-				uint32_t tv = ZEND_RESULT(opline).var;		/* temporary variable */
-				int er;
 
-				er = EG(error_reporting);
-				EG(error_reporting) = 0;
-				if (unary_op(&result, &ZEND_OP1_LITERAL(opline)) != SUCCESS) {
-					EG(error_reporting) = er;
-					break;
-				}
-				EG(error_reporting) = er;
-
-				if (zend_optimizer_replace_by_const(op_array, opline + 1, IS_TMP_VAR, tv, &result)) {
+				if (zend_optimizer_eval_unary_op(&result, opline->opcode, &ZEND_OP1_LITERAL(opline)) == SUCCESS) {
 					literal_dtor(&ZEND_OP1_LITERAL(opline));
-					MAKE_NOP(opline);
-				}
-			}
-			break;
-
-#if 0
-		case ZEND_ADD_STRING:
-		case ZEND_ADD_CHAR:
-			{
-				zend_op *next_op = opline + 1;
-				int requires_conversion = (opline->opcode == ZEND_ADD_CHAR? 1 : 0);
-				size_t final_length = 0;
-				zend_string *str;
-				char *ptr;
-				zend_op *last_op;
-
-				/* There is always a ZEND_RETURN at the end
-				if (next_op>=end) {
-					break;
-				}
-				*/
-				while (next_op->opcode == ZEND_ADD_STRING || next_op->opcode == ZEND_ADD_CHAR) {
-					if (ZEND_RESULT(opline).var != ZEND_RESULT(next_op).var) {
-						break;
-					}
-					if (next_op->opcode == ZEND_ADD_CHAR) {
-						final_length += 1;
-					} else { /* ZEND_ADD_STRING */
-						final_length += Z_STRLEN(ZEND_OP2_LITERAL(next_op));
-					}
-					next_op++;
-				}
-				if (final_length == 0) {
-					break;
-				}
-				last_op = next_op;
-				final_length += (requires_conversion? 1 : Z_STRLEN(ZEND_OP2_LITERAL(opline)));
-				str = zend_string_alloc(final_length, 0);
-				str->len = final_length;
-				ptr = str->val;
-				ptr[final_length] = '\0';
-				if (requires_conversion) { /* ZEND_ADD_CHAR */
-					char chval = (char)Z_LVAL(ZEND_OP2_LITERAL(opline));
-
-					ZVAL_NEW_STR(&ZEND_OP2_LITERAL(opline), str);
-					ptr[0] = chval;
-					opline->opcode = ZEND_ADD_STRING;
-					ptr++;
-				} else { /* ZEND_ADD_STRING */
-					memcpy(ptr, Z_STRVAL(ZEND_OP2_LITERAL(opline)), Z_STRLEN(ZEND_OP2_LITERAL(opline)));
-					ptr += Z_STRLEN(ZEND_OP2_LITERAL(opline));
-					zend_string_release(Z_STR(ZEND_OP2_LITERAL(opline)));
-					ZVAL_NEW_STR(&ZEND_OP2_LITERAL(opline), str);
-				}
-				next_op = opline + 1;
-				while (next_op < last_op) {
-					if (next_op->opcode == ZEND_ADD_STRING) {
-						memcpy(ptr, Z_STRVAL(ZEND_OP2_LITERAL(next_op)), Z_STRLEN(ZEND_OP2_LITERAL(next_op)));
-						ptr += Z_STRLEN(ZEND_OP2_LITERAL(next_op));
-						literal_dtor(&ZEND_OP2_LITERAL(next_op));
-					} else { /* ZEND_ADD_CHAR */
-						*ptr = (char)Z_LVAL(ZEND_OP2_LITERAL(next_op));
-						ptr++;
-					}
-					MAKE_NOP(next_op);
-					next_op++;
-				}
-				if (!((ZEND_OPTIMIZER_PASS_5|ZEND_OPTIMIZER_PASS_10) & OPTIMIZATION_LEVEL)) {
-					/* NOP removal is disabled => insert JMP over NOPs */
-					if (last_op-opline >= 3) { /* If we have more than 2 NOPS then JMP over them */
-						(opline + 1)->opcode = ZEND_JMP;
-						ZEND_OP1(opline + 1).opline_num = last_op - op_array->opcodes; /* that's OK even for ZE2, since opline_num's are resolved in pass 2 later */
+					if (zend_optimizer_replace_by_const(op_array, opline + 1, IS_TMP_VAR, opline->result.var, &result)) {
+						MAKE_NOP(opline);
+					} else {
+						opline->opcode = ZEND_QM_ASSIGN;
+						zend_optimizer_update_op1_const(op_array, opline, &result);
 					}
 				}
 			}
 			break;
-#endif
 
 		case ZEND_FETCH_CONSTANT:
-			if (ZEND_OP2_TYPE(opline) == IS_CONST &&
+			if (opline->op2_type == IS_CONST &&
 				Z_TYPE(ZEND_OP2_LITERAL(opline)) == IS_STRING &&
 				Z_STRLEN(ZEND_OP2_LITERAL(opline)) == sizeof("__COMPILER_HALT_OFFSET__") - 1 &&
 				memcmp(Z_STRVAL(ZEND_OP2_LITERAL(opline)), "__COMPILER_HALT_OFFSET__", sizeof("__COMPILER_HALT_OFFSET__") - 1) == 0) {
@@ -258,21 +143,24 @@ void zend_optimizer_pass1(zend_op_array *op_array, zend_optimizer_ctx *ctx)
 				fake_execute_data.func = (zend_function*)op_array;
 				EG(current_execute_data) = &fake_execute_data;
 				if ((offset = zend_get_constant_str("__COMPILER_HALT_OFFSET__", sizeof("__COMPILER_HALT_OFFSET__") - 1)) != NULL) {
-					uint32_t tv = ZEND_RESULT(opline).var;
 
-					if (zend_optimizer_replace_by_const(op_array, opline, IS_TMP_VAR, tv, offset)) {
-						literal_dtor(&ZEND_OP2_LITERAL(opline));
+					literal_dtor(&ZEND_OP2_LITERAL(opline));
+					if (zend_optimizer_replace_by_const(op_array, opline, IS_TMP_VAR, opline->result.var, offset)) {
 						MAKE_NOP(opline);
+					} else {
+						opline->opcode = ZEND_QM_ASSIGN;
+						opline->extended_value = 0;
+						SET_UNUSED(opline->op2);
+						zend_optimizer_update_op1_const(op_array, opline, offset);
 					}
 				}
 				EG(current_execute_data) = orig_execute_data;
 				break;
 			}
 
-			if (ZEND_OP2_TYPE(opline) == IS_CONST &&
+			if (opline->op2_type == IS_CONST &&
 				Z_TYPE(ZEND_OP2_LITERAL(opline)) == IS_STRING) {
 				/* substitute persistent constants */
-				uint32_t tv = ZEND_RESULT(opline).var;
 				zval c;
 
 				if (!zend_optimizer_get_persistent_constant(Z_STR(ZEND_OP2_LITERAL(opline)), &c, 1)) {
@@ -283,20 +171,25 @@ void zend_optimizer_pass1(zend_op_array *op_array, zend_optimizer_ctx *ctx)
 				if (Z_TYPE(c) == IS_CONSTANT_AST) {
 					break;
 				}
-				if (zend_optimizer_replace_by_const(op_array, opline, IS_TMP_VAR, tv, &c)) {
-					literal_dtor(&ZEND_OP2_LITERAL(opline));
+				literal_dtor(&ZEND_OP2_LITERAL(opline));
+				if (zend_optimizer_replace_by_const(op_array, opline, IS_TMP_VAR, opline->result.var, &c)) {
 					MAKE_NOP(opline);
+				} else {
+					opline->opcode = ZEND_QM_ASSIGN;
+					opline->extended_value = 0;
+					SET_UNUSED(opline->op2);
+					zend_optimizer_update_op1_const(op_array, opline, &c);
 				}
 			}
 			break;
 
 		case ZEND_FETCH_CLASS_CONSTANT:
-			if (ZEND_OP2_TYPE(opline) == IS_CONST &&
+			if (opline->op2_type == IS_CONST &&
 				Z_TYPE(ZEND_OP2_LITERAL(opline)) == IS_STRING) {
 
 				zend_class_entry *ce = NULL;
 
-				if (ZEND_OP1_TYPE(opline) == IS_CONST &&
+				if (opline->op1_type == IS_CONST &&
 			        Z_TYPE(ZEND_OP1_LITERAL(opline)) == IS_STRING) {
 					/* for A::B */
 					if (op_array->scope &&
@@ -314,22 +207,21 @@ void zend_optimizer_pass1(zend_op_array *op_array, zend_optimizer_ctx *ctx)
 						}
 					}
 				} else if (op_array->scope &&
-					ZEND_OP1_TYPE(opline) == IS_UNUSED &&
+					opline->op1_type == IS_UNUSED &&
 					(opline->op1.num & ZEND_FETCH_CLASS_MASK) == ZEND_FETCH_CLASS_SELF) {
 					/* for self::B */
 					ce = op_array->scope;
 				} else if (op_array->scope &&
-					ZEND_OP1_TYPE(opline) == IS_VAR &&
+					opline->op1_type == IS_VAR &&
 					(opline - 1)->opcode == ZEND_FETCH_CLASS &&
-					(ZEND_OP1_TYPE(opline - 1) == IS_UNUSED &&
-					((opline - 1)->extended_value & ZEND_FETCH_CLASS_MASK) == ZEND_FETCH_CLASS_SELF) &&
-					ZEND_RESULT((opline - 1)).var == ZEND_OP1(opline).var) {
+					((opline - 1)->op2_type == IS_UNUSED &&
+					((opline - 1)->op1.num & ZEND_FETCH_CLASS_MASK) == ZEND_FETCH_CLASS_SELF) &&
+					(opline - 1)->result.var == opline->op1.var) {
 					/* for self::B */
 					ce = op_array->scope;
 				}
 
 				if (ce) {
-					uint32_t tv = ZEND_RESULT(opline).var;
 					zend_class_constant *cc;
 					zval *c, t;
 
@@ -338,26 +230,30 @@ void zend_optimizer_pass1(zend_op_array *op_array, zend_optimizer_ctx *ctx)
 						(Z_ACCESS_FLAGS(cc->value) & ZEND_ACC_PPP_MASK) == ZEND_ACC_PUBLIC) {
 						c = &cc->value;
 						if (Z_TYPE_P(c) == IS_CONSTANT_AST) {
-							break;
-						}
-						if (ZEND_IS_CONSTANT_TYPE(Z_TYPE_P(c))) {
-							if (!zend_optimizer_get_persistent_constant(Z_STR_P(c), &t, 1) ||
-							    ZEND_IS_CONSTANT_TYPE(Z_TYPE(t))) {
+							zend_ast *ast = Z_ASTVAL_P(c);
+							if (ast->kind != ZEND_AST_CONSTANT
+							 || !zend_optimizer_get_persistent_constant(zend_ast_get_constant_name(ast), &t, 1)
+							 || Z_TYPE(t) == IS_CONSTANT_AST) {
 								break;
 							}
 						} else {
-							ZVAL_COPY_VALUE(&t, c);
-							zval_copy_ctor(&t);
+							ZVAL_COPY_OR_DUP(&t, c);
 						}
 
-						if (zend_optimizer_replace_by_const(op_array, opline, IS_TMP_VAR, tv, &t)) {
-							if (ZEND_OP1_TYPE(opline) == IS_CONST) {
-								literal_dtor(&ZEND_OP1_LITERAL(opline));
-							} else if (ZEND_OP1_TYPE(opline) == IS_VAR) {
-								MAKE_NOP((opline - 1));
-							}
-							literal_dtor(&ZEND_OP2_LITERAL(opline));
+						if (opline->op1_type == IS_CONST) {
+							literal_dtor(&ZEND_OP1_LITERAL(opline));
+						} else if (opline->op1_type == IS_VAR) {
+							MAKE_NOP((opline - 1));
+						}
+						literal_dtor(&ZEND_OP2_LITERAL(opline));
+
+						if (zend_optimizer_replace_by_const(op_array, opline, IS_TMP_VAR, opline->result.var, &t)) {
 							MAKE_NOP(opline);
+						} else {
+							opline->opcode = ZEND_QM_ASSIGN;
+							opline->extended_value = 0;
+							SET_UNUSED(opline->op2);
+							zend_optimizer_update_op1_const(op_array, opline, &t);
 						}
 					}
 				}
@@ -373,7 +269,7 @@ void zend_optimizer_pass1(zend_op_array *op_array, zend_optimizer_ctx *ctx)
 				send1_opline--;
 			}
 			if (send1_opline->opcode != ZEND_SEND_VAL ||
-			    ZEND_OP1_TYPE(send1_opline) != IS_CONST) {
+			    send1_opline->op1_type != IS_CONST) {
 				/* don't colllect constants after unknown function call */
 				collect_constants = 0;
 				break;
@@ -385,7 +281,7 @@ void zend_optimizer_pass1(zend_op_array *op_array, zend_optimizer_ctx *ctx)
 					send1_opline--;
 				}
 				if (send1_opline->opcode != ZEND_SEND_VAL ||
-				    ZEND_OP1_TYPE(send1_opline) != IS_CONST) {
+				    send1_opline->op1_type != IS_CONST) {
 					/* don't colllect constants after unknown function call */
 					collect_constants = 0;
 					break;
@@ -396,7 +292,7 @@ void zend_optimizer_pass1(zend_op_array *op_array, zend_optimizer_ctx *ctx)
 				init_opline--;
 			}
 			if (init_opline->opcode != ZEND_INIT_FCALL ||
-			    ZEND_OP2_TYPE(init_opline) != IS_CONST ||
+			    init_opline->op2_type != IS_CONST ||
 			    Z_TYPE(ZEND_OP2_LITERAL(init_opline)) != IS_STRING) {
 				/* don't colllect constants after unknown function call */
 				collect_constants = 0;
@@ -436,7 +332,6 @@ void zend_optimizer_pass1(zend_op_array *op_array, zend_optimizer_ctx *ctx)
 			}
 
 			/* pre-evaluate constant functions:
-			   defined(x)
 			   constant(x)
 			   function_exists(x)
 			   is_callable(x)
@@ -470,17 +365,21 @@ void zend_optimizer_pass1(zend_op_array *op_array, zend_optimizer_ctx *ctx)
 						} else {
 							ZVAL_FALSE(&t);
 						}
-						if (zend_optimizer_replace_by_const(op_array, opline + 1, IS_VAR, ZEND_RESULT(opline).var, &t)) {
-							literal_dtor(&ZEND_OP2_LITERAL(init_opline));
-							MAKE_NOP(init_opline);
-							literal_dtor(&ZEND_OP1_LITERAL(send1_opline));
-							MAKE_NOP(send1_opline);
+						literal_dtor(&ZEND_OP2_LITERAL(init_opline));
+						MAKE_NOP(init_opline);
+						literal_dtor(&ZEND_OP1_LITERAL(send1_opline));
+						MAKE_NOP(send1_opline);
+						if (zend_optimizer_replace_by_const(op_array, opline + 1, IS_VAR, opline->result.var, &t)) {
 							MAKE_NOP(opline);
-							zend_string_release(lc_name);
-							break;
+						} else {
+							opline->opcode = ZEND_QM_ASSIGN;
+							opline->extended_value = 0;
+							SET_UNUSED(opline->op2);
+							zend_optimizer_update_op1_const(op_array, opline, &t);
 						}
 					}
-					zend_string_release(lc_name);
+					zend_string_release_ex(lc_name, 0);
+					break;
 				} else if (Z_STRLEN(ZEND_OP2_LITERAL(init_opline)) == sizeof("extension_loaded")-1 &&
 					!memcmp(Z_STRVAL(ZEND_OP2_LITERAL(init_opline)),
 						"extension_loaded", sizeof("extension_loaded")-1) &&
@@ -491,9 +390,9 @@ void zend_optimizer_pass1(zend_op_array *op_array, zend_optimizer_ctx *ctx)
 					zend_module_entry *m = zend_hash_find_ptr(&module_registry,
 							lc_name);
 
-					zend_string_release(lc_name);
+					zend_string_release_ex(lc_name, 0);
 					if (!m) {
-						if (!PG(enable_dl)) {
+						if (PG(enable_dl)) {
 							break;
 						} else {
 							ZVAL_FALSE(&t);
@@ -510,32 +409,19 @@ void zend_optimizer_pass1(zend_op_array *op_array, zend_optimizer_ctx *ctx)
 						}
 					}
 
-					if (zend_optimizer_replace_by_const(op_array, opline + 1, IS_VAR, ZEND_RESULT(opline).var, &t)) {
-						literal_dtor(&ZEND_OP2_LITERAL(init_opline));
-						MAKE_NOP(init_opline);
-						literal_dtor(&ZEND_OP1_LITERAL(send1_opline));
-						MAKE_NOP(send1_opline);
+					literal_dtor(&ZEND_OP2_LITERAL(init_opline));
+					MAKE_NOP(init_opline);
+					literal_dtor(&ZEND_OP1_LITERAL(send1_opline));
+					MAKE_NOP(send1_opline);
+					if (zend_optimizer_replace_by_const(op_array, opline + 1, IS_VAR, opline->result.var, &t)) {
 						MAKE_NOP(opline);
-						break;
-					}
-				} else if (Z_STRLEN(ZEND_OP2_LITERAL(init_opline)) == sizeof("defined")-1 &&
-					!memcmp(Z_STRVAL(ZEND_OP2_LITERAL(init_opline)),
-						"defined", sizeof("defined")-1) &&
-					!zend_optimizer_is_disabled_func("defined", sizeof("defined") - 1)) {
-					zval t;
-
-					if (zend_optimizer_get_persistent_constant(Z_STR(ZEND_OP1_LITERAL(send1_opline)), &t, 0)) {
-
-						ZVAL_TRUE(&t);
-						if (zend_optimizer_replace_by_const(op_array, opline + 1, IS_VAR, ZEND_RESULT(opline).var, &t)) {
-							literal_dtor(&ZEND_OP2_LITERAL(init_opline));
-							MAKE_NOP(init_opline);
-							literal_dtor(&ZEND_OP1_LITERAL(send1_opline));
-							MAKE_NOP(send1_opline);
-							MAKE_NOP(opline);
-							break;
-						}
+					} else {
+						opline->opcode = ZEND_QM_ASSIGN;
+						opline->extended_value = 0;
+						SET_UNUSED(opline->op2);
+						zend_optimizer_update_op1_const(op_array, opline, &t);
 					}
+					break;
 				} else if (Z_STRLEN(ZEND_OP2_LITERAL(init_opline)) == sizeof("constant")-1 &&
 					!memcmp(Z_STRVAL(ZEND_OP2_LITERAL(init_opline)),
 						"constant", sizeof("constant")-1) &&
@@ -543,30 +429,20 @@ void zend_optimizer_pass1(zend_op_array *op_array, zend_optimizer_ctx *ctx)
 					zval t;
 
 					if (zend_optimizer_get_persistent_constant(Z_STR(ZEND_OP1_LITERAL(send1_opline)), &t, 1)) {
-						if (zend_optimizer_replace_by_const(op_array, opline + 1, IS_VAR, ZEND_RESULT(opline).var, &t)) {
-							literal_dtor(&ZEND_OP2_LITERAL(init_opline));
-							MAKE_NOP(init_opline);
-							literal_dtor(&ZEND_OP1_LITERAL(send1_opline));
-							MAKE_NOP(send1_opline);
-							MAKE_NOP(opline);
-							break;
-						}
-					}
-				} else if ((CG(compiler_options) & ZEND_COMPILE_NO_BUILTIN_STRLEN) == 0 &&
-					Z_STRLEN(ZEND_OP2_LITERAL(init_opline)) == sizeof("strlen") - 1 &&
-					!memcmp(Z_STRVAL(ZEND_OP2_LITERAL(init_opline)), "strlen", sizeof("strlen") - 1) &&
-					!zend_optimizer_is_disabled_func("strlen", sizeof("strlen") - 1)) {
-					zval t;
-
-					ZVAL_LONG(&t, Z_STRLEN(ZEND_OP1_LITERAL(send1_opline)));
-					if (zend_optimizer_replace_by_const(op_array, opline + 1, IS_VAR, ZEND_RESULT(opline).var, &t)) {
 						literal_dtor(&ZEND_OP2_LITERAL(init_opline));
 						MAKE_NOP(init_opline);
 						literal_dtor(&ZEND_OP1_LITERAL(send1_opline));
 						MAKE_NOP(send1_opline);
-						MAKE_NOP(opline);
-						break;
+						if (zend_optimizer_replace_by_const(op_array, opline + 1, IS_VAR, opline->result.var, &t)) {
+							MAKE_NOP(opline);
+						} else {
+							opline->opcode = ZEND_QM_ASSIGN;
+							opline->extended_value = 0;
+							SET_UNUSED(opline->op2);
+							zend_optimizer_update_op1_const(op_array, opline, &t);
+						}
 					}
+					break;
 				/* dirname(IS_CONST/IS_STRING) -> IS_CONST/IS_STRING */
 				} else if (Z_STRLEN(ZEND_OP2_LITERAL(init_opline)) == sizeof("dirname")-1 &&
 					!memcmp(Z_STRVAL(ZEND_OP2_LITERAL(init_opline)),
@@ -579,17 +455,22 @@ void zend_optimizer_pass1(zend_op_array *op_array, zend_optimizer_ctx *ctx)
 						zval t;
 
 						ZVAL_STR(&t, dirname);
-						if (zend_optimizer_replace_by_const(op_array, opline + 1, IS_VAR, ZEND_RESULT(opline).var, &t)) {
-							literal_dtor(&ZEND_OP2_LITERAL(init_opline));
-							MAKE_NOP(init_opline);
-							literal_dtor(&ZEND_OP1_LITERAL(send1_opline));
-							MAKE_NOP(send1_opline);
+						literal_dtor(&ZEND_OP2_LITERAL(init_opline));
+						MAKE_NOP(init_opline);
+						literal_dtor(&ZEND_OP1_LITERAL(send1_opline));
+						MAKE_NOP(send1_opline);
+						if (zend_optimizer_replace_by_const(op_array, opline + 1, IS_VAR, opline->result.var, &t)) {
 							MAKE_NOP(opline);
-							break;
+						} else {
+							opline->opcode = ZEND_QM_ASSIGN;
+							opline->extended_value = 0;
+							SET_UNUSED(opline->op2);
+							zend_optimizer_update_op1_const(op_array, opline, &t);
 						}
 					} else {
-						zend_string_release(dirname);
+						zend_string_release_ex(dirname, 0);
 					}
+					break;
 				}
 			}
 			/* don't colllect constants after any other function call */
@@ -597,28 +478,33 @@ void zend_optimizer_pass1(zend_op_array *op_array, zend_optimizer_ctx *ctx)
 			break;
 		}
 		case ZEND_STRLEN:
-			if (ZEND_OP1_TYPE(opline) == IS_CONST &&
-			    Z_TYPE(ZEND_OP1_LITERAL(opline)) == IS_STRING) {
+			if (opline->op1_type == IS_CONST) {
 				zval t;
 
-				ZVAL_LONG(&t, Z_STRLEN(ZEND_OP1_LITERAL(opline)));
-				if (zend_optimizer_replace_by_const(op_array, opline + 1, IS_TMP_VAR, ZEND_RESULT(opline).var, &t)) {
+				if (zend_optimizer_eval_strlen(&t, &ZEND_OP1_LITERAL(opline)) == SUCCESS) {
 					literal_dtor(&ZEND_OP1_LITERAL(opline));
-					MAKE_NOP(opline);
+					if (zend_optimizer_replace_by_const(op_array, opline + 1, IS_TMP_VAR, opline->result.var, &t)) {
+						MAKE_NOP(opline);
+					} else {
+						opline->opcode = ZEND_QM_ASSIGN;
+						zend_optimizer_update_op1_const(op_array, opline, &t);
+					}
 				}
 			}
 			break;
 		case ZEND_DEFINED:
 			{
 				zval c;
-				uint32_t tv = ZEND_RESULT(opline).var;
 				if (!zend_optimizer_get_persistent_constant(Z_STR(ZEND_OP1_LITERAL(opline)), &c, 0)) {
 					break;
 				}
 				ZVAL_TRUE(&c);
-				if (zend_optimizer_replace_by_const(op_array, opline, IS_TMP_VAR, tv, &c)) {
-					literal_dtor(&ZEND_OP1_LITERAL(opline));
+				literal_dtor(&ZEND_OP1_LITERAL(opline));
+				if (zend_optimizer_replace_by_const(op_array, opline, IS_TMP_VAR, opline->result.var, &c)) {
 					MAKE_NOP(opline);
+				} else {
+					opline->opcode = ZEND_QM_ASSIGN;
+					zend_optimizer_update_op1_const(op_array, opline, &c);
 				}
 			}
 			break;
@@ -629,6 +515,52 @@ void zend_optimizer_pass1(zend_op_array *op_array, zend_optimizer_ctx *ctx)
 				zend_optimizer_collect_constant(ctx, &ZEND_OP1_LITERAL(opline), &ZEND_OP2_LITERAL(opline));
 			}
 			break;
+#if 0
+		/* see ext/opcache/tests/bug78961.phpt */
+//		case ZEND_FETCH_R:
+		case ZEND_FETCH_W:
+//		case ZEND_FETCH_RW:
+		case ZEND_FETCH_IS:
+//		case ZEND_FETCH_FUNC_ARG:
+		case ZEND_FETCH_UNSET:
+			/* convert FETCH $GLOBALS (global), FETCH_DIM $x into FETCH $x (global) */
+			if ((opline->extended_value & ZEND_FETCH_GLOBAL) != 0 &&
+			    opline->op1_type == IS_CONST &&
+			    Z_TYPE(ZEND_OP1_LITERAL(opline)) == IS_STRING &&
+			    zend_string_equals_literal(Z_STR(ZEND_OP1_LITERAL(opline)), "GLOBALS") &&
+			    ((opline + 1)->opcode == opline->opcode + 1 ||
+			     ((opline + 1)->opcode == ZEND_UNSET_DIM &&
+			      opline->opcode == ZEND_FETCH_UNSET) ||
+			     ((opline + 1)->opcode == ZEND_ISSET_ISEMPTY_DIM_OBJ &&
+			      opline->opcode == ZEND_FETCH_IS)) &&
+			    (opline + 1)->op1_type == opline->result_type &&
+			    (opline + 1)->op1.var == opline->result.var &&
+			    ((opline + 1)->op2_type != IS_CONST ||
+			     Z_TYPE(ZEND_OP2_LITERAL(opline + 1)) < IS_ARRAY)) {
+
+				if ((opline + 1)->opcode == ZEND_UNSET_DIM) {
+					(opline + 1)->opcode = ZEND_UNSET_VAR;
+					(opline + 1)->extended_value = ZEND_FETCH_GLOBAL;
+				} else if ((opline + 1)->opcode == ZEND_ISSET_ISEMPTY_DIM_OBJ) {
+					(opline + 1)->opcode = ZEND_ISSET_ISEMPTY_VAR;
+					(opline + 1)->extended_value |= ZEND_FETCH_GLOBAL;
+				} else {
+					(opline + 1)->opcode = opline->opcode;
+					(opline + 1)->extended_value = ZEND_FETCH_GLOBAL;
+				}
+				(opline + 1)->op1_type = (opline + 1)->op2_type;
+				(opline + 1)->op1 = (opline + 1)->op2;
+				if ((opline + 1)->op1_type == IS_CONST &&
+				    Z_TYPE(ZEND_OP1_LITERAL(opline + 1)) != IS_STRING) {
+
+					convert_to_string(&ZEND_OP1_LITERAL(opline + 1));
+					zend_string_hash_val(Z_STR(ZEND_OP1_LITERAL(opline + 1)));
+				}
+				SET_UNUSED((opline + 1)->op2);
+				MAKE_NOP(opline);
+			}
+			break;
+#endif
 
 		case ZEND_RETURN:
 		case ZEND_RETURN_BY_REF:
